

function result = CLBC_GQ(varargin) % image,radius,mode


% Check number of input arguments.
error(nargchk(1,3,nargin));

image=varargin{1};
d_image=double(image);

if nargin==1
    spoints=[-1 -1; -1 0; -1 1; 0 -1; -0 1; 1 -1; 1 0; 1 1];
    neighbors=8;
    mode='h';
end

if (nargin == 2) && (length(varargin{2}) == 1)
    error('Input arguments');
end

if (nargin > 2) && (length(varargin{2}) == 1)
    radius=varargin{2};
    neighbors=radius*8; %default neighbors number  set by experimental results 

    spoints=zeros(neighbors,2);

    % Angle step.
    a = 2*pi/neighbors;
    
    for i = 1:neighbors
        spoints(i,1) = -radius*sin((i-1)*a);
        spoints(i,2) = radius*cos((i-1)*a);
    end
    
    
    if(nargin == 3)
        mode=varargin{3};
    else
        mode='h';
    end
end

% Determine the dimensions of the input image.
[ysize xsize] = size(image);



miny=min(spoints(:,1));
maxy=max(spoints(:,1));
minx=min(spoints(:,2));
maxx=max(spoints(:,2));

% Block size, each LBC code is computed within a block of size bsizey*bsizex
bsizey=ceil(max(maxy,0))-floor(min(miny,0))+1;
bsizex=ceil(max(maxx,0))-floor(min(minx,0))+1;

% Coordinates of origin (0,0) in the block
origy=1-floor(min(miny,0));
origx=1-floor(min(minx,0));

% Minimum allowed size for the input image depends
% on the radius of the used LBC operator.
if(xsize < bsizex || ysize < bsizey)
  error('Too small input image');
end

% Calculate dx and dy;
dx = xsize - bsizex;
dy = ysize - bsizey;

% Fill the center pixel matrix C.
C = image(origy:origy+dy,origx:origx+dx);
d_C = double(C);

bins = 2^neighbors;

% Initialize the result matrix with zeros.
CS=zeros(dy+1,dx+1);
CM=zeros(dy+1,dx+1);
CC=zeros(dy+1,dx+1);
NCS=zeros(dy+1,dx+1);
S11=zeros(dy+1,dx+1);
S12=zeros(dy+1,dx+1);
S01=zeros(dy+1,dx+1);
S02=zeros(dy+1,dx+1);
CM_h=zeros(dy+1,dx+1);
CM_d=zeros(dy+1,dx+1);
CM_t=zeros(dy+1,dx+1);
CM_f=zeros(dy+1,dx+1);
%Compute the LBC code image

for i = 1:neighbors
  y = spoints(i,1)+origy;
  x = spoints(i,2)+origx;
  % Calculate floors, ceils and rounds for the x and y.
  fy = floor(y); cy = ceil(y); ry = round(y);
  fx = floor(x); cx = ceil(x); rx = round(x);
  % Check if interpolation is needed.
  if (abs(x - rx) < 1e-6) && (abs(y - ry) < 1e-6)
    % Interpolation is not needed, use original datatypes
    N = image(ry:ry+dy,rx:rx+dx);
    D{i} = N >= C;
    ND{i} = N< C;
    Diff{i} = abs(N-C);
%     Diffs{i}= N-C;
    MeanDiff(i) = mean(mean(Diff{i}));
  else
    % Interpolation needed, use double type images 
    ty = y - fy;
    tx = x - fx;

    % Calculate the interpolation weights.
    w1 = (1 - tx) * (1 - ty);
    w2 =      tx  * (1 - ty);
    w3 = (1 - tx) *      ty ;
    w4 =      tx  *      ty ;
    % Compute interpolated pixel values
    N = w1*d_image(fy:fy+dy,fx:fx+dx) + w2*d_image(fy:fy+dy,cx:cx+dx) + ...
        w3*d_image(cy:cy+dy,fx:fx+dx) + w4*d_image(cy:cy+dy,cx:cx+dx);
    D{i} = N >= d_C; 
    ND{i} = N < d_C;
    Diff{i} = abs(N-d_C);
%     Diffs{i}= N-d_C;
    MeanDiff(i) = mean(mean(Diff{i}));
  end  
end
% Difference threshold for CLBC_M
DiffThreshold = mean(MeanDiff);
DiffT_half=DiffThreshold/2;
DiffT_double=DiffThreshold*2;
DiffT_trible=DiffThreshold*3;
DiffT_fourth=DiffThreshold*4;
DiffT_max=DiffThreshold*6;
% compute CLBC_S and CLBC_M
for i=1:neighbors
  % Update the result matrix.
  CS = CS + D{i};
  NCS = NCS +ND{i};
  CM = CM + (Diff{i}>=DiffThreshold);
  CM_h=CM_h + (Diff{i}>=DiffT_half);
  CM_d=CM_d + (Diff{i}>=DiffT_double);
  CM_t=CM_t + (Diff{i}>=DiffT_trible);
  CM_f=CM_f + (Diff{i}>=DiffT_fourth);
%   S12 = S12 + (Diffs{i}>=DiffThreshold);
%   S11= S11 + (Diffs{i}>=0);
%   S01= S01 + (Diffs{i}<0);
%   S12 = S12 + (Diffs{i}< -DiffThreshold);
end
% S11=S11-S12;
% S01=S01-S02;

% CLBC_C
CQlevel=6; %2~9
if CQlevel==2
    CC = d_C>=mean(d_image(:));
else
    CC = GrayQuantization(d_C,CQlevel,'hist');%mode can be 'eq' 'hist' 'mean'
end

if (strcmp(mode,'h') || strcmp(mode,'hist') || strcmp(mode,'nh'))
    % Return with LBC histogram if mode equals 'hist'.
    % In our demo-codes, 'test' mode is used for feature-fusion
%     CSH=hist(CS(:),0:neighbors);
%     CM=hist(CM(:),0:neighbors);
    if radius==2
        CS = GrayQuantization(CS,9,'eq');
        CM = GrayQuantization(CM,9,'eq');
        CM_t = GrayQuantization(CM_t,9,'eq');

        SM=1000.*CC+100.*CS+10.*CM+CM_t;
        SMH=hist(SM(:),0:9999);
        
    elseif radius==3

%             CS = GrayQuantization(CS,25,'eq');
%             CM = GrayQuantization(CM,25,'eq');
%             CM_t = GrayQuantization(CM_t,25,'eq');
% 
%             SM=100000.*CC+10000.*CS+100.*CM+CM_t;
%             SMH=hist(SM(:),0:3999999);
% 
%             SMH=sparse(SMH);

            CS = GrayQuantization(CS,9,'eq');
            CM = GrayQuantization(CM,9,'eq');
            CM_t = GrayQuantization(CM_t,9,'eq');

            SM=1000.*CC+100.*CS+10.*CM+CM_t;
            SMH=hist(SM(:),0:9999);

    else
        SM=1000.*CC+100.*CS+10.*CM+CM_t;
        SMH=hist(SM(:),0:9999);    

%         SM=100.*CC+10.*CS+1.*CM; % N(3)
%         SMH=hist(SM(:),0:999); 


    end

    result=SMH;
end






