function result = LQCH(varargin) % image,radius,mode
% version 0.2 by Z.Y.  
% version 0.1 (CLBC_GQ)2012-12-04
% the quantization thresholds are also changed.

% Check number of input arguments.
error(nargchk(1,3,nargin));

image=varargin{1};
d_image=double(image);

if nargin==1
    spoints=[-1 -1; -1 0; -1 1; 0 -1; -0 1; 1 -1; 1 0; 1 1];
    neighbors=8;
    mode='h';
end

if (nargin == 2) && (length(varargin{2}) == 1)
    error('Input arguments');
end

if (nargin > 2) && (length(varargin{2}) == 1)
    radius=varargin{2};
    neighbors=radius*8; %default neighbors number  set by experimental results 

    spoints=zeros(neighbors,2);

    % Angle step.
    a = 2*pi/neighbors;
    
    for i = 1:neighbors
        spoints(i,1) = -radius*sin((i-1)*a);
        spoints(i,2) = radius*cos((i-1)*a);
    end
    
    
    if(nargin == 3)
        mode=varargin{3};
    else
        mode='h';
    end
end

% Determine the dimensions of the input image.
[ysize xsize] = size(image);

miny=min(spoints(:,1));
maxy=max(spoints(:,1));
minx=min(spoints(:,2));
maxx=max(spoints(:,2));

bsizey=ceil(max(maxy,0))-floor(min(miny,0))+1;
bsizex=ceil(max(maxx,0))-floor(min(minx,0))+1;

% Coordinates of origin (0,0) in the block
origy=1-floor(min(miny,0));
origx=1-floor(min(minx,0));

% Minimum allowed size for the input image depends
% on the radius of the used LBC operator.
if(xsize < bsizex || ysize < bsizey)
  error('Too small input image');
end

% Calculate dx and dy;
dx = xsize - bsizex;
dy = ysize - bsizey;

% Fill the center pixel matrix C.
C = image(origy:origy+dy,origx:origx+dx);
d_C = double(C);

bins = 2^neighbors;

% Initialize the result matrix with zeros.
CS=zeros(dy+1,dx+1);
CM=zeros(dy+1,dx+1);
CM_2=zeros(dy+1,dx+1);
CM_d=zeros(dy+1,dx+1);
CM_t=zeros(dy+1,dx+1);
CM_f=zeros(dy+1,dx+1);
CM_4=zeros(dy+1,dx+1);
CM_3=zeros(dy+1,dx+1);
CM_10=zeros(dy+1,dx+1);

%change 1 image to 9 sub-images
DD=zeros(dy+1,dx+1,neighbors+1);
DD=double(DD);
DD(:,:,neighbors+1)=d_C;
for i = 1:neighbors
  y = spoints(i,1)+origy;
  x = spoints(i,2)+origx;
  % Calculate floors, ceils and rounds for the x and y.
  fy = floor(y); cy = ceil(y); ry = round(y);
  fx = floor(x); cx = ceil(x); rx = round(x);
  % Check if interpolation is needed.
  if (abs(x - rx) < 1e-6) && (abs(y - ry) < 1e-6)
    % Interpolation is not needed, use original datatypes
    N = image(ry:ry+dy,rx:rx+dx);
    DD(:,:,i)=double(N);
  else
    % Interpolation needed, use double type images 
    ty = y - fy;
    tx = x - fx;

    % Calculate the interpolation weights.
    w1 = (1 - tx) * (1 - ty);
    w2 =      tx  * (1 - ty);
    w3 = (1 - tx) *      ty ;
    w4 =      tx  *      ty ;
    % Compute interpolated pixel values
    N = w1*d_image(fy:fy+dy,fx:fx+dx) + w2*d_image(fy:fy+dy,cx:cx+dx) + ...
        w3*d_image(cy:cy+dy,fx:fx+dx) + w4*d_image(cy:cy+dy,cx:cx+dx);
    DD(:,:,i)=N;
  end  
end

%Get the minimum pixels in Local neighbor set
Cmin=zeros(dy+1,dx+1);
Cmean=zeros(dy+1,dx+1);
Cmin=min(DD,[],3);
Cmean=mean(DD,3);
% disp(size(Cmin));

%Get gray difference value
for j=1:neighbors+1
%     Diff_Sign{j}=DD(:,:,j)-Cmean;
    Diff{j} = abs(DD(:,:,j)-Cmin);
    MeanDiff(j) = mean(mean(Diff{j}));
end

% Difference thresholds for Grayscale Difference
% These paraments can be adjusted
DiffThreshold = mean(MeanDiff);
%  disp(DiffThreshold);
DiffT_half=DiffThreshold/2;
DiffT_quarter=DiffThreshold/4;
DiffT_onethird=DiffThreshold/3;
DiffT_deci=DiffThreshold/10;
DiffT_double=DiffThreshold*2;
DiffT_trible=DiffThreshold*3;
DiffT_fourth=DiffThreshold*4;

for i=1:neighbors+1
  % Update the result matrix.
%   CS = CS + (Diff_Sign{i}>=0);
CS = CS + (Diff{i}>0);
  CM = CM + (Diff{i}>=DiffThreshold);
  CM_2=CM_2 + (Diff{i}>=DiffT_half);
  CM_4=CM_4 + (Diff{i}>=DiffT_quarter);
  CM_3=CM_3 + (Diff{i}>=DiffT_onethird);
  CM_10=CM_10 + (Diff{i}>=DiffT_deci);
  CM_d=CM_d + (Diff{i}>=DiffT_double);
  CM_t=CM_t + (Diff{i}>=DiffT_trible);
  CM_f=CM_f + (Diff{i}>=DiffT_fourth);

end

Nlevel=4; %neighbor quantization levels
Clevel=6; %central quantization levels
% quantization of Local base pixel
if Clevel==2
    CC = d_C>=mean(d_image(:));
%     CC = Cmin>=mean(d_image(:));
else
    CC = GrayQuantization(Cmean,Clevel,'hist');%eq hist mean
end



if (strcmp(mode,'h') || strcmp(mode,'hist') || strcmp(mode,'nh'))
    % Return with LBC histogram if mode equals 'hist'.
    % In our demo-codes, 'test' mode is used for feature-fusion
    if Nlevel==3
            if radius==2
                CS = GrayQuantization(CS,9,'eq');
                CM = GrayQuantization(CM,9,'eq');
                CM_t = GrayQuantization(CM_t,9,'eq');
                SM=100.*CC+10.*CS+CM_t; 
                SMH=hist(SM(:),0:Clevel*100);
                 %%% without quantization
                SM=10000.*CC+100.*CS+CM_t;              
                SMH=hist(SM(:),0:Clevel*10000); 
            elseif radius==3
                CS = GrayQuantization(CS,9,'eq');
                CM = GrayQuantization(CM,9,'eq');
                CM_t = GrayQuantization(CM_t,9,'eq');
                SM=100.*CC+10.*CS+CM_t; 
                SMH=hist(SM(:),0:Clevel*100);
            end
            SM=100.*CC+10.*CS+CM;              
            SMH=hist(SM(:),0:Clevel*100);    
    elseif Nlevel==4
            if radius==2
                CS = GrayQuantization(CS,9,'eq');
                CM = GrayQuantization(CM,9,'eq');
                CM_t = GrayQuantization(CM_t,9,'eq');     
                SM=1000.*CC+100.*CS+10.*CM+CM_t;
                SMH=hist(SM(:),0:Clevel*1000); 
            end
            if radius==3
                CS = GrayQuantization(CS,19,'eq');
                CM = GrayQuantization(CM,19,'eq');
                CM_t = GrayQuantization(CM_t,19,'eq');
                SM=10000.*CC+1000.*CS+10.*CM+CM_t;
                SMH=hist(SM(:),0:Clevel*10000);
            end
            SM=1000.*CC+100.*CS+10.*CM+CM_t;          
            SMH=hist(SM(:),0:Clevel*1000); 
%     elseif Nlevel==5
        
    else
        SM=1000.*CC+100.*CS+10.*CM+CM_t;
        SMH=hist(SM(:),0:9999);          
    end

    result=SMH;
end


end





